#ifndef FD_xtuner_h_
  #include "xtuner.h"
#endif

extern flags_t flags;
extern xtuner_data_t xtuner_data;
extern Display *dpy;
extern GC canvasGC;
extern FD_xtuner *mainui;

/* ======================================================================== */
/* Function to send frequency data 					    */
/* ======================================================================== */
void r_out(int v, int n)
{
  while (n--)
  {
    if (v&1)
    {
      OUTW(5,xtuner_data.rport);
      OUTW(5,xtuner_data.rport);
      OUTW(7,xtuner_data.rport);
      OUTW(7,xtuner_data.rport);
    }
    else
    {
      OUTW(1,xtuner_data.rport);
      OUTW(1,xtuner_data.rport);
      OUTW(3,xtuner_data.rport);
      OUTW(3,xtuner_data.rport);
    }
    v>>=1;
  }
}
/* ======================================================================== */
/* Turns the radio card on and tunes to current frequency		    */
/* ======================================================================== */
void radioon(int ch, int p2)
{
  update();
 
  r_out(ch,16);
  r_out(p2,8);

  usleep(1000);
  OUTB(0,xtuner_data.rport);

  usleep(50000);
  OUTB(0xc8,xtuner_data.rport);
}

/* ====================================================================== */
/* Turn off the radio card						  */
/* ====================================================================== */
void radiooff(void)
{ 
  /*r_out(FCODE(88.0),16);
  r_out(0xa0,8);

  OUTB(0,xtuner_data.rport);
  OUTB(0xc0,xtuner_data.rport);*/
  
  OUTB(0x00,xtuner_data.rport);
  OUTB(0x00,xtuner_data.rport);
}

/* ======================================================================== */
/* Increases the radio cards volume level				    */
/* ======================================================================== */ 
void volumeup(void)
{
  OUTB(0x88,xtuner_data.rport);
  usleep(100000);
  OUTB(0xc8,xtuner_data.rport);
}

/* ======================================================================== */
/* Lowers the radio cards volume level			 		    */
/* ======================================================================== */
void volumedn(void)
{
  OUTB(0x48,xtuner_data.rport);
  usleep(100000);
  OUTB(0xc8,xtuner_data.rport);
}

/* ======================================================================== */
/* Initialize various flags, variables, and io capabilites                  */
/* ======================================================================== */ 
void init_radio()
{
  int i, error_flag;
  FILE *fdin;

  /* Default radio variables */
  xtuner_data.on_mins   = 0;
  xtuner_data.off_mins  = 0;
  xtuner_data.on_hours  = 0;
  xtuner_data.off_hours = 0; 
  xtuner_data.freq      = 98.7;
  xtuner_data.vol_level = 0;
  
  /* Default flags */
  flags.disp_type_f  = NORM;
  flags.radio_f      = OFF;
  flags.set_f        = OFF;
  flags.mute_f       = OFF;
  flags.auto_f       = OFF;
  flags.preset_f     = OFF;
  flags.mixer_f      = FALSE; 
  
  /* Defualt presets */
  *xtuner_data.MemCh =
  (
    93.7,
    107.9,
    105.5,
    103.3,
    98.7,
    106.3,
    92.7,
    103.2,
    104.3,
    102.3
  );
  
  /* Set IO permissions for this process */
  #if (defined(BSD) && (BSD >= 199306))
  if( (fpiopl = open( "/dev/io", O_RDONLY ) < 0) ) 
  {
    fprintf(stderr, "XTuner: failed to gain IO privledges.\n");
    exit(1);
  }
  #else
  if(ioperm(xtuner_data.rport,2,1) < 0)
  {
    fprintf(stderr, "Xtuner: failed to gain IO privledges. \n");
    exit(1);
  }
  #endif
  
  radiooff();
  
  /* read in the saved presets */
  read_file();
} 

/* ======================================================================= */
/* Determines if the current frequency has a strong signal       */
/* ======================================================================= */
int is_tuned(void)
{
  int x;

  OUTB(0xf8,xtuner_data.rport);
  usleep(200000);              /* any faster and it won't work? */
  x = inb(xtuner_data.rport);
  usleep(100000);
  OUTB(0xe8,xtuner_data.rport);

  /* if x == ff not tuned, if x == fd then tuned */

  if (( x & 255) != 0x0ff)
  {
    flags.tuned_f = TRUE;
    return TRUE;
  }
  else
  {
    flags.tuned_f = FALSE; 
    return FALSE; 
  }
}
